package app

import (
	"github.com/goccy/go-json"
	"github.com/gofiber/fiber/v2"
	"github.com/spf13/viper"
	"gofiber.study.skcks.cn/common/config"
	"gofiber.study.skcks.cn/common/errorx"
	"gofiber.study.skcks.cn/common/logger"
	"gofiber.study.skcks.cn/common/utils"
	"gofiber.study.skcks.cn/controller"
	"gofiber.study.skcks.cn/global"
)

var App *fiber.App

func Run() {
	err := viper.Unmarshal(global.Config)
	if err != nil {
		logger.Log.Fatalf("配置文件解析失败: %s", err)
	}

	utils.MainAppExec(func() {
		logger.Log.Debugf("\n%#v", global.Config)
	})

	reloadRedis(global.Config)
	reloadDataSources(global.Config)

	App = reloadApp(global.Config)

	if err := App.Listen(global.Config.Server.Addr); err != nil {
		logger.Log.Fatalf("[x] [Fiber] 致命错误: %s", err)
	}

}

func reloadApp(c *config.BasicConfig) *fiber.App {
	if App != nil {
		_ = App.Shutdown()
	}

	app := fiber.New(fiber.Config{
		JSONEncoder:   json.Marshal,
		JSONDecoder:   json.Unmarshal,
		Prefork:       c.Server.PreFork,
		CaseSensitive: c.Server.CaseSensitive,
		StrictRouting: c.Server.StrictRouting,
		ServerHeader:  c.Server.ServerHeader,
		ErrorHandler: func(ctx *fiber.Ctx, err error) error {
			code := fiber.StatusInternalServerError

			if e, ok := err.(*fiber.Error); ok {
				code = e.Code
			}

			err = ctx.Status(code).JSON(errorx.NewDefaultError(err.Error()))
			if err != nil {
				return ctx.Status(fiber.StatusInternalServerError).SendString("Internal Server Error")
			}

			return nil
		},
	})

	if c.Server.EnableRoutesMsg {
		routes(app)
	}

	controller.SwaggerHandler(app)
	controller.RegisterController(app)
	controller.HelloWorld(app)
	controller.ErrorHandler(app)

	app.Hooks().OnListen(func() error {
		utils.MainAppExec(func() {
			logger.Log.Infoln("[√] [服务启动完成]")
		})
		return nil
	})

	return app
}
